import subprocess
import logging


def try_cmd(*args, **kwargs):
    kwargs["check"] = True

    try:
        completed_process = cmd(*args, **kwargs)
    except subprocess.CalledProcessError as e:
        msg = f"Command '{e.cmd}' returned non-zero exit status {e.returncode}."
        if e.stdout or e.stderr:
            msg += "\n"
        if e.stdout:
            msg += f"\nHere is its stdout:\n{e.stdout.decode()}"
        if e.stderr:
            msg += f"\nHere is its stderr:\n{e.stderr.decode()}"
        raise RuntimeError(msg)

    return completed_process


def cmd(*args, **kwargs):
    if "shell" not in kwargs:
        kwargs["shell"] = True
    if "executable" not in kwargs:
        kwargs["executable"] = "/bin/bash"

    logging.debug(f"$ {args[0]}")

    # pylint: disable=subprocess-run-check
    completed_process = subprocess.run(*args, **kwargs)
    return completed_process



from pathlib import Path

import drvn.installer._utils as utils


def is_cloned():
    path = _get_repository_path()
    return path.is_dir()


def clone():
    cwd = _get_repository_path().parent
    utils.try_cmd(
        "git clone https://github.com/hallgrimur1471/.dotfiles.git", cwd=cwd
    )


def get_path(path_relative_to_repository):
    repository_path = _get_repository_path()
    return repository_path / path_relative_to_repository


def _get_repository_path():
    return Path("~/.dotfiles").expanduser()



import logging
import argparse

import drvn.installer.installer as installer
import drvn.installer._logging as drvn_logger


def main():
    args = _parse_arguments()

    if args.verbose:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO
    drvn_logger.configure(log_level)

    if args.install:
        software_to_install = args.install.split(",")
        for software in software_to_install:
            installer.install(software, args.with_drvn_configs)


def _parse_arguments():
    parser = argparse.ArgumentParser()
    parser.description = (
        "Installers for various software with options "
        + "on additionally installing custom configs."
    )
    parser.add_argument(
        "-i",
        "--install",
        metavar="SOFTWARE_LIST",
        help="Comma seperated list of software to install. "
        + f"Possible values: {','.join(installer.get_installable_software())}",
    )
    parser.add_argument(
        "-c",
        "--with-drvn-configs",
        action="store_true",
        help="Additionally install drvn's custom configs with the software "
        + "(drvn's .vimrc with vim, etc)",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Enables printing of debug statements",
    )
    arguments = parser.parse_args()
    return arguments



import logging


def configure(log_level):
    logging.basicConfig(
        format="[%(asctime)s.%(msecs)03d %(levelname)s %(module)s]: %(message)s",
        datefmt="%H:%M:%S",
        level=log_level,
    )



import sys
import logging
from pathlib import Path

import drvn.installer._utils as utils
import drvn.installer._dotfile_repository as dotfile_repository


def install(is_install_drvn_configs):
    logging.info("Installing ultimate_vim ...")
    _install_prerequisites()
    _install_ultimate_vim()
    _install_extra_plugins()
    if is_install_drvn_configs:
        _install_drvn_configs()


def _install_prerequisites():
    utils.try_cmd("sudo apt-get install -y cmake")

    python = _get_current_python_interpreter()
    utils.try_cmd(f"{python} -m pip install python-language-server")
    utils.try_cmd(f"{python} -m pip install mypy")


def _install_ultimate_vim():
    ultimate_vim_path = Path("~/.vim_runtime").expanduser()
    if ultimate_vim_path.is_dir():
        logging.debug(
            "Looks like ultimate_vim is already cloned, "
            + "skipping cloning ..."
        )
    else:
        utils.try_cmd(
            "git clone --depth=1 "
            + f"https://github.com/amix/vimrc.git {ultimate_vim_path}"
        )
    utils.try_cmd("sh ~/.vim_runtime/install_awesome_vimrc.sh")


def _install_extra_plugins():
    _install_you_complete_me()
    _install_more_color_schemes()


def _install_you_complete_me():
    ycm_path_str = "~/.vim_runtime/my_plugins/YouCompleteMe"
    ycm_path = Path(ycm_path_str).expanduser()
    if not ycm_path.is_dir():
        utils.try_cmd(
            "cd ~/.vim_runtime/my_plugins && "
            + "git clone https://github.com/ycm-core/YouCompleteMe.git && "
            + "cd YouCompleteMe && "
            + "git submodule update --init --recursive"
        )
    else:
        logging.debug(
            "Looks like YouCompleteMe is already cloned, "
            + "skipping cloning ..."
        )

    python = _get_current_python_interpreter()
    utils.try_cmd(
        "cd ~/.vim_runtime/my_plugins/YouCompleteMe && "
        + f"{python} install.py --clang-completer"
    )

def _install_more_color_schemes():
    plugins_path = Path("~/.vim_runtime/sources_forked").expanduser()
    colorschemes_path = plugins_path / "vim-colorschemes"
    if not colorschemes_path.is_dir():
        utils.try_cmd("git clone https://github.com/flazz/vim-colorschemes.git")
    else:
        logging.debug("Looks like vim-colorschemes is already cloned, "+"skipping cloning ...")

def _install_drvn_configs():
    if not dotfile_repository.is_cloned():
        dotfile_repository.clone()
    _symlink_configs_to_dotfile_repository()


def _symlink_configs_to_dotfile_repository():
    dotfile_path = Path(".vim_runtime/my_configs.vim")
    real_path = dotfile_repository.get_path(dotfile_path)
    symlink_path = Path("~").expanduser() / dotfile_path
    utils.cmd(f"rm {symlink_path}")
    utils.try_cmd(f"ln -s '{real_path}' '{symlink_path}'")


def _get_current_python_interpreter():
    major = sys.version_info.major
    minor = sys.version_info.minor
    return f"python{major}.{minor}"



# pylint: disable=no-self-use, unused-argument, redefined-outer-name
import logging
from pathlib import Path

import pytest

import drvn.installer._utils as utils


@pytest.fixture(scope="class")
def workspace():
    workspace_path = _set_up_workspace()
    yield workspace_path
    _tear_down_workspace()


class TestDrvnInstallerScript:
    def test_help_exits_with_returncode_zero(self):
        _assert_returncode_zero("drvn_installer --help")

    def test_normal_run_exits_with_returncode_zero(self, workspace):
        _assert_returncode_zero("drvn_installer", cwd=workspace)

    # TODO: do some install testing in a docker container

_assert_returncode_zero = utils.try_cmd

def _set_up_workspace():
    workspace_path = _get_workspace_path()
    logging.debug("Setting up integration test workspace ...")
    utils.try_cmd(f"mkdir -p {workspace_path}")
    return workspace_path


def _tear_down_workspace():
    workspace_path = _get_workspace_path()
    logging.debug("Tearing down integration test workspace ...")
    utils.try_cmd(f"rm -rf {workspace_path}")


def _get_workspace_path():
    workspace_path = Path("/tmp/python_installer/integration_workspace")
    return workspace_path



# pylint: disable=invalid-name
import setuptools

with open("README.md", "r") as readme_file:
    long_description = readme_file.read()

setuptools.setup(
    name="drvn.installer",
    author="Hallgrimur David Egilsson",
    author_email="hallgrimur1471@gmail.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hallgrimur1471/drvn_installer",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.7",
    package_dir={"": "src"},
    packages=setuptools.find_namespace_packages(where="src"),
    package_data={"": ["*"]},
    entry_points={
        "console_scripts": [
            "drvn_installer = drvn.installer._script:main"
        ]
    },
    install_requires=[],
)
